/*
 *  Copyright (C) 2002-2007  The DOSBox Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Local Debug Function */

/* $Id: debug_inc.h,v 1.11 2007/01/08 19:45:39 qbix79 Exp $ */

#include <curses.h>
#include "mem.h"

#define PAIR_BLACK_BLUE 1
#define PAIR_BYELLOW_BLACK 2
#define PAIR_GREEN_BLACK 3
#define PAIR_BLACK_GREY 4
#define PAIR_GREY_RED 5

#define DEBUG_WINDOW_SIZE	50

void DBGUI_StartUp(void);

struct DBGBlock {
	WINDOW * win_main;					/* The Main Window */
	WINDOW * win_reg;					/* Register Window */
	WINDOW * win_data;					/* Data Output window */
	WINDOW * win_code;					/* Disassembly/Debug point Window */
	WINDOW * win_command;
	WINDOW * win_var;					/* Variable Window */
	WINDOW * win_out;					/* Text Output Window */
	Bit32u active_win;					/* Current active window */
	Bit32u global_mask;					/* Current msgmask */
};


struct DASMLine {
	Bit32u pc;
	char dasm[80];
	PhysPt ea;
	Bit16u easeg;
	Bit32u eaoff;
};

extern DBGBlock dbg;

/* Local Debug Stuff */
Bitu DasmI386(char* buffer, PhysPt pc, Bitu cur_ip, bool bit32);
int  DasmLastOperandSize(void);

class CDebugView
{
public:
	CDebugView(int init_min,int init_size) {
		min_size=init_min; size=init_size;  
		view_num=last;
		views[last++]=this;
		on();
	};

	int size;
private:
	int view_num;
	int min_size;
	int view_on;
public:
	bool is_on(void) { return view_on;};
	void on(void)  { 
		view_on=1;
		calc_free_size();
		if(free_size>=0) return;

		view_on=0;		
		calc_free_size();
		for(int i=1;i<last;i++)	{
			int n= (i+view_num)% last ;
			if(views[n]->is_on()) {
				if( (views[n]->size-views[n]->min_size) + free_size >= size)	{					
					view_on=1;
					views[n]->size=free_size+views[n]->size-size;
					return;
				}			
			}
		}

		for(int i=1;i<last;i++)	{
			int n= (i+view_num)% last ;
			if(views[n]->is_on()) {
				views[n]->size=views[n]->min_size;
			}
		}
		calc_free_size();
		size=free_size;
		view_on=1;
	};

	void off(void) { view_on=0;};
	void toggle(void) { if(is_on()) off(); else on(); }		// view_on to toggle may abstract to SWITCH class, then derived it

	void set_size(int siz) { size=siz; }

	static bool layoutChange;
private:
	static int main_size;
	static int free_size;
	static CDebugView * views[32];
	static int last;
	
	void calc_free_size(void)	{
		int total_size=0;
		for(int i=0;i<last;i++)	{			
			if(views[i]->is_on()) {
				if(views[i]->size < views[i]->min_size) views[i]->size=views[i]->min_size;
				total_size+=views[i]->size;
				total_size++;	// title				
			}
		}
		free_size=main_size-total_size;		
	};
};

extern CDebugView vReg;
extern CDebugView vData;
extern CDebugView vCode;
extern CDebugView vCommand;
extern CDebugView vVar;
extern CDebugView vOut;


